#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version = 0.50

#pragma hide=1
#include "Image Tools Main"

#define DEMO

#ifdef DEMO
Static Constant k_version = -0.5
#else
#include "Radial Profiler Calculator"
Static Constant k_version = 0.5
#endif

// locations for Image Tools Package
Static StrConstant k_fullpackageFolder = "root:Packages:imgT"
Static StrConstant k_imgDisplay = "imgT_display"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"
Static StrConstant k_panelName = "imgT_panel"

// location for this package
Static StrConstant k_datafolder = "RadialProfiles"

// panel and graph sizes
Static Constant pwidth = 280
Static Constant pheight = 545
Static Constant gwidth = 300

//// menu placement
Menu "Packages"
	SubMenu "Image Processing"
		"Radial Profiler", /Q, init_RadialProfiler()
	end
end

// *** STRUCTURES

// radial profiler settings
Static Structure S_RadialProfilerParams
	variable fedge
	variable mask
	variable cx
	variable cy
	variable rinx
	variable rdouter
	variable circular
	variable aspyx
	variable dx
	variable faveover
	variable method
	variable overlay
endStructure

Function Sf_ReadRadialProfilerParams(srp)
	STRUCT S_RadialProfilerParams &srp

	ControlInfo/W=RP_Inputs setvarEdge
	srp.fedge = v_value
	ControlInfo/W=RP_Inputs popupRPMask
	srp.mask = v_value

	ControlInfo/W=RP_Inputs setvarcx
	srp.cx = v_value
	ControlInfo/W=RP_Inputs setvarcy
	srp.cy = v_value
	ControlInfo/W=RP_Inputs setvarmir
	srp.rinx = v_value
	ControlInfo/W=RP_Inputs setvarmor
	srp.rdouter = v_value
	ControlInfo/W=RP_Inputs checkCircular
	srp.circular = v_value
	ControlInfo/W=RP_Inputs valdispaspyx
	srp.aspyx = v_value

	ControlInfo/W=RP_Inputs setvaraveo
	srp.faveover = v_value
	ControlInfo/W=RP_Inputs popupMethod
	srp.method = v_value

	ControlInfo/W=RP_Inputs popupQuadrant
	srp.overlay = v_value
	
	return 0
end

Function Sf_InitRadialProfilerParams(srp)
	STRUCT S_RadialProfilerParams &srp

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string wdf = "root:" + cwdfolder
	variable left, right, top, bottom, minr, dx

	wave/SDFR=$wdf simg = $cwfile
	left = IndexToScale(simg,0,0)
	right = IndexToScale(simg,inf,0)
	top = IndexToScale(simg,0,1)
	bottom = IndexToScale(simg,inf,1)
	dx = DimDelta(simg,0)

	srp.fedge = 0.01
	srp.mask = 1
	srp.cx = round((right + left)/2)
	srp.cy = round((bottom + top)/2)
	minr = min((right - left),(bottom - top))
	srp.rinx = round(minr/(4*dx))
	srp.rdouter = 0.10
	srp.circular = 1
	srp.aspyx = 1
	srp.dx = dx
	
	return 0
end	

// list of profile mask methods
Function/S f_RPMasks()

	string rtstr = "None;"
	
	STRUCT S_MarqueeCoordinates mq
	Sf_GetMQCoordinates(mq)

	if (mq.hasmarquee == 1)
		rtstr += "x<[ ]>x; [>x<] ;"
	endif
	
	return rtstr
end

// initialize the package
Function init_RadialProfiler()

	// just show the existing panel
	DoWindow/F RProfile
	if (v_flag == 1)
		return 0
	endif
	
	// cannot create without Image Tools
	DoWindow/F $k_imgDisplay
	if (v_flag == 0)
		DoAlert/T="No Image Tools Window", 0, "You must have an Image Tools Window active to run this package."
		return 0
	endif
	
	// check dimensions
	if (init_RPConfirmImage() < 0)
		DoAlert/T="Incorrect XY Dimension Sizes", 0, "This profiler only works on images with the same (x,y) scaling."
		return 0
	endif
	
	// set up the data folder
	init_RPDataFolder()

	STRUCT S_RadialProfilerParams srp
	
	// inititalize the rings
	init_RPRings(srp)
	
	// create and show panel
	init_RPPanel(srp)
	
	return 0
end

// check dimension sizes
Function init_RPConfirmImage()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf = "root:" + cwdfolder
	
	// cannot work on images with mis-matched dimension sizes
	wave/SDFR=$wdf simg = $cwfile
	if (DimDelta(simg,0) != DimDelta(simg,1))
		return -1
	endif
	
	return 0
end

// create the datafolder when needed
Function init_RPDataFolder()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder

	string wdf, ddf
	variable dx, dy
	
	wdf = "root:" + cwdfolder
	ddf = wdf + ":" + k_datafolder
	
	if (!DataFolderExists(ddf))
		DFREF cdf = GetDataFolderDFR()
		SetDataFolder $wdf
		NewDataFolder/S $k_datafolder
		SetDataFolder cdf
	endif

	return 0
end

// create the first rings
Function init_RPRings(srp)
	STRUCT S_RadialProfilerParams &srp
	
	Sf_InitRadialProfilerParams(srp)
	Sf_StoreRings(0,0,0,0,srp.cx,srp.cy,srp.rinx,srp.rinx,0,rdouter=srp.rdouter)
	RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)
	RP_SetOuter(srp.rdouter,srp.circular)

	return 0
end

// create and show the panel
Function init_RPPanel(srp)
	STRUCT S_RadialProfilerParams &srp

	string vstr = "v " + num2str(k_version)	
	variable voffset = 20, hoffset = 0
	
	Display/W=(100,50,gwidth+100,pheight+50)/N=RProfile
	SetWindow RProfile hook(rprofile)=Hf_RProfile
	NewPanel/EXT=0/HOST=#/W=(0,0,pwidth,pheight)/N=RP_Inputs as "Radial Profiler"
	ModifyPanel fixedSize=1, noEdit=1
	
	DefaultGuiFont/W=RP_Inputs all={"_IgorMedium",14,0}, button={"_IgorMedium",16,0}, panel={"_IgorMedium",12,0}
	
	SetDrawEnv fsize=10
	DrawText pwidth-40, voffset, vstr
	
	// RING SETUP
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Rings"

	Button buttonautosetrings,pos={pwidth-60,voffset},size={50,25},proc=RP_ButtonProc,title="Find"
	
	voffset+=20
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset+5, "Auto-Locate on Blob"
	
	voffset+=10
	
	SetVariable setvarEdge,pos={20,voffset},size={100,20},proc=RP_SetVarProc,title="Edge", disable=2
	SetVariable setvarEdge,format="%3.2f",limits={0.01,1,0.01},value= _NUM:0.01

	PopupMenu popupRPMask,pos={140,voffset},size={100,25},title="Mask",disable=2
	PopupMenu popupRPMask,mode=1,popvalue="None",value= #"f_RPMasks()"
	
	voffset+=40
		
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset+5, "Position Manually"
	
	voffset+=10
	
	SetVariable setvarcx,pos={20,voffset},size={110,20},proc=RP_SetVarProc,title="<-c->", disable=2
	SetVariable setvarcx,format="%4.0f",limits={-100000,100000,10},value=_NUM:srp.cx
	
	SetVariable setvarcy,pos={150,voffset},size={110,20},proc=RP_SetVarProc,title="∧|c|∨", disable=2
	SetVariable setvarcy,format="%4.0f",limits={-100000,100000,10},value=_NUM:srp.cy
	
	voffset+=25
	
	SetVariable setvarmir,pos={20,voffset},size={135,20},proc=RP_SetVarProc,title="\\[0Inner r\\Bx\\M", disable=2
	SetVariable setvarmir,format="%4.0f",limits={0,100000,10},value=_NUM:srp.rinx
		
	CheckBox checkCircular pos={pwidth-80,voffset+2},title="Circular", value=srp.circular, proc=RP_CheckProc, disable=2
	
	voffset+=25

	SetVariable setvarmor,pos={20,voffset},size={135,20},proc=RP_SetVarProc,title="\\[0Outer ∆r\\Bx\\M", disable=2
	SetVariable setvarmor,format="%3.2f",limits={-1,2,0.01},value=_NUM:srp.rdouter
	
	//voffset+=10

	ValDisplay valdispAspYX,pos={pwidth-105,voffset},size={90,25},title="||/--", disable=2
	ValDisplay valdispAspYX, format="%3.3f",limits={0,0,0},value= _NUM:0

	//Button buttonsetrings,pos={pwidth-60,voffset},size={50,25},proc=RP_ButtonProc,title="Set"
	
	voffset+=30
	
	PopupMenu popupoptimize,pos={15,voffset},size={140,25},title="Optimize",fStyle=2,proc=RP_PopMenuProc, disable=2
	PopupMenu popupoptimize,mode=1,popvalue="Manually",value= #"\"Manually;Smallest Circle;CoM Image;CoM Threshold;CoM Filled;\""

	Button buttonclearrings,pos={pwidth-60,voffset},size={50,25},proc=RP_ButtonProc,title="Clear"
	
	// PROFILE CALCULATIONS
	
	voffset+=45
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Profile Graphs"

	voffset+=25
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset, "Radial Method"

	Button buttonanalyze,pos={pwidth-85,voffset-15},size={75,25},proc=RP_ButtonProc,title="Analyze", disable=2

	voffset+=5
	
	PopupMenu popupmethod,pos={20,voffset},size={120,25},title="",proc=RP_PopMenuProc, disable=2
	PopupMenu popupmethod,mode=1,popvalue="Pixels - Linear",value= #"\"Pixels - Linear;Pixels - Quadratic;Line Profile - Circular;\""

	voffset+=45
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset, "Rows/Columns"
	
	SetVariable setvaraveo,pos={pwidth-110,voffset-18},size={95,25},proc=RP_SetVarProc,title="f\Bave\M", disable=2
	SetVariable setvaraveo,format="%3.2f",limits={-0.1,0.1,0.01},value=_NUM:0

	voffset+=25

	// RESULTS
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Results"
	
	voffset+=20
	
	SetDrawEnv fsize=14,fstyle=2
	DrawText 15, voffset+5, "Distribution"
	
	voffset+=10

	ValDisplay valdisprmp,pos={20,voffset},size={100,25},title="R\Bmp",format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprave,pos={20,voffset},size={100,25},title="R\Bave",format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprrms,pos={20,voffset},size={100,25},title="R\Brms", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprg,pos={20,voffset},size={100,25},title="R\Bg", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdispru50,pos={20,voffset},size={100,25},title="R\BU50", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprfwhm,pos={20,voffset},size={100,25},title="R\Bfwhm", format="%4.4g", disable=2

	voffset-=(5*25+10)
	hoffset = 160
	
	SetDrawEnv fsize=14,fstyle=2
	DrawText hoffset, voffset+5, "Blob"
	
	CheckBox checkShowBlob pos={hoffset+40,voffset-13},title="Show",proc=RP_CheckProc, value=0, disable=2
	
	voffset+=10

	ValDisplay valdispUAD,pos={hoffset,voffset},size={100,25},format="%4.4g",title="U\BAD", disable=2

	voffset+=25

	ValDisplay valdispFUniform,pos={hoffset,voffset},size={100,25},format="%3.4f",title="f\BU", disable=2

	voffset+=25

	ValDisplay valdispBlobA,pos={hoffset,voffset},size={90,25},format="%4.0f",title="A", disable=2

	voffset+=25
	ValDisplay valdispBlobP,pos={hoffset,voffset},size={90,25},format="%4.0f",title="P", disable=2

	voffset+=25

	ValDisplay valdispBlobC,pos={hoffset,voffset},size={90,25},format="%3.1f",title="C", disable=2

	voffset+=30

	Button buttonsave, pos={pwidth-90,voffset},size={80,25},proc=RP_ButtonProc,title="Save", disable=2

	voffset+=25
	
	PopupMenu popupquadrant,pos={15,voffset},size={120,25},title="Quadrant", disable=2
	PopupMenu popupquadrant,mode=1,value= #"\"None;Separate;Overlay;\"",proc=RP_PopMenuProc

	Button buttonpublish,pos={pwidth-90,voffset},size={80,25},proc=RP_ButtonProc,title="Publish", disable=2

	RP_ClearValDisplays()
		
	return 0
end

Function RP_ClearValDisplays()

	ValDisplay valdisprmp, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdisprave, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdisprrms, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdisprg, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdisprU50, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdisprfwhm, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdispUAD, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdispFUniform, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdispBlobA, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdispBlobP, win=RP_Inputs, value=_NUM:NaN
	ValDisplay valdispBlobC, win=RP_Inputs, value=_NUM:NaN

	return 0
end

Function Hf_RProfile(hs)
	STRUCT WMWinHookStruct &hs

	Variable hookResult = 0

	switch(hs.eventCode)
		case 2:		// kill
			DoWindow/F $k_imgdisplay
			if (v_flag)
				FDrawOval(0,0,0,0,"innerring",kill=1)
				FDrawOval(0,0,0,0,"outerring",kill=1)
				FPutCenter(0,0,0)
				RemoveImage/W=$k_imgdisplay/Z RP_quadrant
				DoWindow/K/Z RP_Blob
			endif
			hookResult = 1
			break
	endswitch

	return hookResult
end

// ***
// STRUCTURES

// get rings

Static Function/wave Sf_GetRings(ioro)
	variable ioro

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	
	wave/SDFR=$wdf/Z ringi, ringo
	make/N=4/D/FREE ring

	if (!waveexists(ringi))
		ring = NaN
		return ring
	endif

	switch(ioro)
		case 1:	// inner
			ring[0] = ringi[4]
			ring[1] = ringi[5]
			ring[2] = ringi[6]
			ring[3] = ringi[7]
			break
		case 2:	// outer
			ring[0] = ringo[4]
			ring[1] = ringo[5]
			ring[2] = ringo[6]
			ring[3] = ringo[7]
			break
	endswitch
	
	return ring
end

//store rings
Static Function Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,ring,[rdouter])
	variable left, top, right, bottom, cx, cy, rx, ry, ring, rdouter

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string idf = "root:" + cwdfolder
	string wdf = idf + ":" + k_datafolder
	
	if (!DataFolderExists(wdf))
		DFREF cdf = GetDataFolderDFR()
		SetDataFolder $idf
		NewDataFolder $k_datafolder
		SetDataFolder cdf
	endif
	
	wave/SDFR=$wdf/Z ringi, ringo
	if (waveexists(ringi)==0)
		DFREF cdf = GetdataFolderDFR()
		SetDataFolder $idf
		NewDataFolder/O/S $k_datafolder
		make/N=8 ringi, ringo
		SetDataFolder cdf
	endif
	wave/SDFR=$wdf ringi, ringo
	redimension/N=8 ringi, ringo

	switch(ring)
		case -1:		// update inner
			ringi[6] = rx
			ringi[7] = ry
			ringi[0] = ringi[4] - rx
			ringi[1] = ringi[5] - ry
			ringi[2] = ringi[4] + rx
			ringi[3] = ringi[5] + ry
			break
		case 0:		// initialize
			ringi[0] =  cx - rx
			ringi[1] = cy - ry
			ringi[2] = cx + rx
			ringi[3] = cy + ry	
			ringi[4] = cx
			ringi[5] = cy
			ringi[6] = rx
			ringi[7] = ry
			ringo[0] = cx - rx*(1 + rdouter)
			ringo[1] = cy - ry*(1 + rdouter)
			ringo[2] = cx + rx*(1 + rdouter)
			ringo[3] = cy + ry	*(1 + rdouter)
			ringo[4] = cx
			ringo[5] = cy
			ringo[6] = rx*(1 + rdouter)
			ringo[7] = ry*(1 + rdouter)
			break
		case 1:		// inner
			ringi[0] = left
			ringi[1] = top
			ringi[2] = right
			ringi[3] = bottom	
			ringi[4] = cx
			ringi[5] = cy
			ringi[6] = rx
			ringi[7] = ry
			break
		case 2:		// outer
			ringo[0] = left
			ringo[1] = top
			ringo[2] = right
			ringo[3] = bottom	
			ringo[4] = cx
			ringo[5] = cy
			ringo[6] = rx
			ringo[7] = ry
			break
	endswitch
	return 0
end

// set the target ring
// fedge = fraction edge to find cut on threshold
// cirular = restrict to circular only
// mask = mask marquee region
// returns asperity index

Function RP_SetTarget(fedge, circular, mask)
	variable fedge, circular, mask

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	STRUCT S_MarqueeCoordinates smq
		
	string wdf, swave, thwave
	variable axo, axd, ayo, ayd
	variable cplane, nump, zeroxo, zeroxf, zeroyo, zeroyf
	variable widthx, widthy, cx, cy, rx, ry, wmax
	variable left, top, right, bottom, aspyx, osize = 0
	
	if (f_isThreshold())
		thwave = cwfile
		swave = ReplaceString("_th",cwfile,"")
	else
		if (f_ThresholdExists()==1)
			swave = cwfile
			thwave = swave + "_th"
		else
			DoAlert/T="Image Processing", 0, "No threshold exists for this image. Set one first."
			return 0
		endif
	endif
	
	wdf = "root:" + cwdfolder
	wave/SDFR=$wdf srimg = $swave
	wave/SDFR=$wdf timg = $thwave
	if (f_IsStack())
		cplane = f_PlaneNumber()
		MatrixOP/FREE simg = srimg[][][cplane]
		MatrixOP/FREE thimg = timg[][][cplane]
		duplicate/FREE thimg thximg
	else
		duplicate/FREE srimg, simg
		duplicate/FREE timg, thimg, thximg
	endif

	switch(mask)
		case 2:		// mask outside
			Sf_GetMQCoordinates(smq)
			left = ScaletoIndex(thimg,smq.pgleft,0)
			right = ScaletoIndex(thimg,smq.pgright,0)
			top = ScaletoIndex(thimg,smq.pgtop,1)
			bottom = ScaletoIndex(thimg,smq.pgbottom,1)		
			thimg = 0
			thimg[left,right][top,bottom] = thximg
			break
		case 3:		// mask inside
			Sf_GetMQCoordinates(smq)
			left = ScaletoIndex(thimg,smq.pgleft,0)
			right = ScaletoIndex(thimg,smq.pgright,0)
			top = ScaletoIndex(thimg,smq.pgtop,1)
			bottom = ScaletoIndex(thimg,smq.pgbottom,1)		
//			if (smq.pgleft < 0)
//				smq.pgleft = 0
//			endif
//			if (smq.pgtop < 0)
//				smq.pgtop = 0
//			endif
//			wmax = DimSize(simg,0) - 1
//			if (smq.pgright > wmax)
//				smq.pgright = wmax
//			endif
//			wmax = DimSize(simg,1) - 1
//			if (smq.pgbottom > wmax)
//				smq.pgbottom = wmax
//			endif
			thimg[left,right][top,bottom] = 0
			break
	endswitch
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	NewDataFolder/O/S $k_datafolder
	
	// do rows
	MatrixOP/O sRows = sumRows(simg)
	// set scale for rows
	axo = DimOffset(simg,0)
	axd = DimDelta(simg,0)
	SetScale/P x, axo,axd, sRows
	
	MatrixOP/FREE tRows = sumRows(thimg)
	SetScale/P x, axo,axd, tRows	

	MatrixOP/FREE mRows = maxRows(simg)
	
	nump = numpnts(tRows)
	WaveStats/Q/M=1 tRows
	FindLevel/EDGE=1/Q tRows, fedge*V_max
	zeroxo = round(v_LevelX)
	FindLevel/EDGE=2/R=[nump-1,0]/Q tRows, fedge*V_max
	zeroxf = round(v_LevelX)
	rx = (zeroxf - zeroxo)/2
	widthx = 2*rx
	cx = round((zeroxf + zeroxo)/2)
	
	// do columns
	MatrixOP/O sCols = sumCols(simg)^t
	// set scale for columns
	ayo = DimOffset(simg,1)
	ayd = DimDelta(simg,1)
	SetScale/P x, ayo,ayd, sCols
	MatrixOP/FREE tCols = sumCols(thimg)^t
	SetScale/P x, axo,axd, tCols
		
	MatrixOP/FREE mCols = maxCols(simg)^t

	nump = numpnts(tCols)
	WaveStats/Q/M=1 tCols
	FindLevel/EDGE=1/Q tCols, fedge*V_max
	zeroyo = round(v_LevelX)
	FindLevel/EDGE=2/R=[nump-1,0]/Q tCols, fedge*V_max
	zeroyf = round(v_LevelX)
	ry = (zeroyf - zeroyo)/2
	widthy = 2*ry
	cy = round((zeroyf + zeroyo)/2)
	aspyx = widthy/widthx
	
	wmax = max(widthx, widthy)
	if (circular)
		widthx = wmax
		widthy = wmax
		rx = ceil(wmax/2)
		ry = rx
	endif
	
	left = round(cx - (widthx/2))
	right = round(left + widthx)
	
	// restrict left
	if (left < axo)
		right += left
		left = axo
		widthx = right - left
		osize = 1
	endif
	//restrict right
	wmax = DimSize(simg,0) - 1
	if (right > wmax)
		left += wmax - right
		right = wmax
		widthx = right - left
		osize = 1
	endif
	top = round(cy - (widthy/2))
	bottom = round(top + widthy)
	// restrict top
	if (top < ayo)
		bottom += top
		top = 0
		widthy = bottom - top
		osize = 1
	endif
	// restrict bobbom
	wmax = DimSize(simg,1) - 1
	if (bottom > wmax)
		top += wmax - bottom
		bottom = wmax
		widthy = bottom - top
		osize = 1
	endif
	
	if (osize && circular)
		wmax = min(widthx,widthy)
		widthx = wmax
		widthy = wmax
		left = round(cx - widthx/2)
		right = round(cx + widthx/2)
		top = round(cy - widthy/2)
		bottom = round(cy + widthy/2)
	endif
	
	// show stuff
	FPutCenter(cx, cy, 1)
	FDrawOval(left, top, right, bottom, "innerring")
	Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,1)
	//DoWindow/F RProfile
	if (WinType("RProfile")==1)
		ReplaceWave/W=RProfile allinCDF
	endif
	SetDataFolder cdf
	
	return aspyx
end

// snip profiles
// input values are x values of ww
Function FSnipProfile(ww,xskeep,xekeep)
	wave ww
	variable xskeep, xekeep
	
	variable po, pf, pm
	
	po = x2pnt(ww,xskeep) - 1
	pf = x2pnt(ww,xekeep) + 1
	pm = numpnts(ww)-1
	
	// clip left
	if (po > 0)
		ww[0,po] = NaN
	endif
	
	// clip right
	if (pf < pm)	
		ww[pf,pm] = NaN
	endif
	
	return 0
end

// put/remove the center cursor
Function FPutCenter(cx, cy, show)
	variable cx, cy, show
	
	string cImage = ""
	switch(show)
		case 0:		// kill
			DoWindow/F $k_imgDisplay
			if (v_flag)
				Cursor/W=$k_imgDisplay/K A
			endif
			break
		case 1:
			cImage = f_CurrentImage()
			Cursor/W=$k_imgDisplay/I/H=1/S=0/T=3 A $cImage cx, cy
			break
	endswitch
	return 0
end

// draw an oval
Function FDrawOval(left,top,right,bottom,oname,[kill])
	variable left, top, right, bottom, kill
	string oname
	
	if (!ParamIsDefault(kill))
		DoWindow/F $k_imgDisplay
		if (v_flag)
			SetDrawLayer/W=$k_imgDisplay ProgFront
			DrawAction/W=$k_imgDisplay getgroup=$oname, delete
		endif
		return 0
	endif
		
	SetDrawLayer/W=$k_imgDisplay ProgFront
	DrawAction/W=$k_imgDisplay getgroup=$oname, delete, begininsert
	SetDrawEnv/W=$k_imgDisplay gstart, gname=$oname
	SetDrawEnv/W=$k_imgDisplay xcoord=top, ycoord=left
	SetDrawEnv/W=$k_imgDisplay linefgc=(65535,65535,0),fillpat=0,linethick=3
	DrawOval/W=$k_imgDisplay left,top,right,bottom
	SetDrawEnv/W=$k_imgDisplay gstop
	SetDrawLayer/W=$k_imgDisplay UserFront
	return 0
end

// create row + colume profiles
// ao - average over
// rprofile - outer radius?
Function RP_MakeRowColProfiles(ao)
	variable ao

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	
	variable xleft, xtop, xright, xbottom
	variable xlimitl, xlimitt, xlimitr, xlimitb
	variable wpxmax, wpymax, npo
	variable dx, dy, wx, wy, wxo, wxf, wyo, wyf
	variable rave, lave, mbl, bbl
	variable maxR, maxC
	
	// set waves to work
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	
	variable cx, cy
	wave ringo, sCols, sRows
	
	// outer ring values set profile limits
	xleft = ringo[0]
	xtop = ringo[1]
	xright = ringo[2]
	xbottom = ringo[3]
	cx = ringo[4]
	cy = ringo[5]
	dx = DimDelta(sRows,0)
	wx = xright - xleft + dx
	dy = DimDelta(sCols,0)
	wy = xbottom - xtop + dx
	
	// number of points to add or subtract to averaging
	wpxmax = numpnts(sRows)
	wpymax = numpnts(sCols)
	if (ao < 0)
		npo = -round(min(-ao*wpxmax,-ao*wpymax))
	else
		npo = round(min(ao*wpxmax,ao*wpymax))
	endif

	// ROWS
	// right side
	wxf = pnt2x(sRows,wpxmax-1)
	xlimitr = xright + npo*dx
	xlimitr = xlimitr > wxf ? wxf : xlimitr
	WaveStats/Q/R=(xright,xlimitr) sRows
	rave = V_avg	
	// left side
	wxo = pnt2x(sRows,0)
	xlimitl = xleft - npo*dx
	xlimitl = xlimitl < wxo ? wxo : xlimitl
	WaveStats/Q/R=(xlimitl,xleft) sRows
	lave = V_avg
	// baseline
	wx = xright - xleft
	mbl = (rave - lave)/wx
	bbl = lave - mbl*xleft
	Duplicate/O sRows zRows
	Redimension/D zRows
	zRows = sRows - (mbl*x + bbl)
	
	Duplicate/FREE zRows zsRows
	if (xleft > 0)
		zsRows[0,xleft]=0
	endif
	if (xright < wxf)
		zsRows[xright,wxf] = 0
	endif
	Sort/R zsRows, zsRows
	SetScale/P x, 0, 1, zsRows
	maxR = sum(zsRows,0,(abs(npo)))/(abs(npo)+1)
	zRows /= maxR
	FSnipProfile(zRows, xleft, xright)
	
	// COLS
	// bottom
	wyf = pnt2x(sCols,wpymax-1)
	xlimitb = xbottom + npo*dy
	xlimitb = xlimitb > wyf ? wyf : xlimitb
	WaveStats/Q/R=(xbottom,xlimitb) sCols
	rave = V_avg
	// top
	wyo = pnt2x(sCols,0)
	xlimitt = xtop - npo*dy
	xlimitt = xlimitt < wyo ? wyo : xlimitt
	WaveStats/Q/R=(xlimitt,xtop) sCols
	lave = V_avg
	// baseline
	wy = xbottom - xtop
	mbl = (rave - lave)/wy
	bbl = rave - mbl*xbottom
	Duplicate/O sCols zCols
	Redimension/D zCols
	zCols = sCols - (mbl*x + bbl)
	
	Duplicate/FREE zCols zsCols
	if (xtop > 0)
		zsCols[0,xtop]=0
	endif
	if (xbottom < wyf)
		zsCols[xbottom,wyf] = 0
	endif
	Sort/R zsCols, zscols
	SetScale/P x, 0, 1, zsCols
	maxC = sum(zsCols,0,abs(npo))/(abs(npo)+1)
	zCols /= maxC
	FSnipProfile(zCols, xtop, xbottom)
	
	SetDataFolder cdf
	
	return 0
end

// update the row/column plot
Function RP_UpdateRowColPlot([clear])
	variable clear

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":RadialProfiles"
	variable crsL, crsT, crsR, crsB
	
	// cannot update when data folder does not exist
	if (!DataFolderExists(wdf))
		return 0
	endif
	
	wave/SDFR=$wdf/Z zCols, zRows, ringi
	
	if (!waveexists(zCols))
		return 0
	endif
	
	crsL = ringi[0]
	crsT = ringi[1]
	crsR = ringi[2]
	crsB = ringi[3]
	
	duplicate/FREE zCols zmC
	duplicate/FREE zRows zmR
	
	if (!ParamIsDefault(clear))
		ModifyGraph/W=RProfile/Z hideTrace(zCols)=1, hideTrace(zRows)=1
		return 0
	endif
	
	if (ItemsInList(TraceNameList("RProfile",";",1))!=0)
		ReplaceWave/W=RProfile trace=zCols, zCols
		ReplaceWave/W=RProfile trace=zRows, zRows
		ModifyGraph/Z/W=RProfile hideTrace(zCols)=0, hideTrace(zRows)=0
		Tag/W=RProfile/C/N=LEFT/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsL,"L"
		Tag/W=RProfile/C/N=RIGHT/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsR,"R"
		Tag/W=RProfile/C/N=TOP/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsT,"T"
		Tag/W=RProfile/C/N=BOTTOM/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsB,"B"
	else
		AppendtoGraph/W=RProfile zCols,zRows
		ModifyGraph/W=RProfile rgb(zCols)=(1,4,52428),rgb(zRows)=(52428,1,1), zero(left)=4
		ModifyGraph/W=RProfile mirror(left)=2,mirror(bottom)=2,nticks(left)=8
		ModifyGraph/W=RProfile lblPos(left)=73,lblPos(bottom)=47
		ModifyGraph/W=RProfile axisEnab(left)={0,0.49},mirror(bottom)=0,axOffset(left)=-2
		SetAxis/W=RProfile left -0.2,1.2
		TextBox/W=RProfile/C/N=profiles/F=0/A=RC/X=5/Y=-10 "\\K(1,4,52428)blue >|\r\\K(52428,1,1)red   \\[0v\\X0_"
		ModifyGraph/Z/W=RProfile hideTrace(zCols)=0, hideTrace(zRows)=0
		Tag/W=RProfile/C/N=LEFT/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsL,"L"
		Tag/W=RProfile/C/N=RIGHT/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsR,"R"
		Tag/W=RProfile/C/N=TOP/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsT,"T"
		Tag/W=RProfile/C/N=BOTTOM/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsB,"B"
	endif
	
	return 0
end

// ***
// PANEL CONTROLS

Function RP_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	STRUCT S_RadialProfilerParams srp
	variable aspyx, rmax
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			strswitch(sva.ctrlName)
				case "setvarEdge":		// edge
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					aspyx = RP_SetTarget(srp.fedge,srp.circular,srp.mask)
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ClearValDisplays()
					ValDisplay valdispAspYX, win=RP_Inputs, value=_NUM:aspyx
					Button buttonpublish, win=RP_Inputs, disable=2
					PopupMenu popupquadrant, win=RP_Inputs, disable=2
					rmax = max(Sf_GetRings(1)[2],Sf_GetRings(1)[3])
					SetVariable setvarmir, win=RP_Inputs, value=_NUM:rmax
					break
				case "setvarcx":
				case "setvarcy":
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)		
					RP_SetOuter(srp.rdouter,srp.circular)				
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					break
				case "setvarmir":		// inner ring position
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy,sva.dval, srp.aspyx, srp.circular)		
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ClearValDisplays()	
					//ValDisplay valdispAspYX, win=RP_Inputs, value=_NUM:aspyx, disable=0
					Button buttonpublish, win=RP_Inputs, disable=2
					PopupMenu popupquadrant, win=RP_Inputs, disable=2
#ifndef DEMO
					RP_PlotNormalizedProfile(clear=1)
#endif
					break
				case "setvarmor":		// outer ring position
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ClearValDisplays()
					Button buttonpublish, win=RP_Inputs, disable=2
					PopupMenu popupquadrant, win=RP_Inputs, disable=2
#ifndef DEMO
					RP_PlotNormalizedProfile(clear=1)
#endif
					break
				case "setvaraveo":		// average over for baseline
					// reset the profiles
					RP_MakeRowColProfiles(sva.dval)
					RP_UpdateRowColPlot()
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
end

Function RP_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	STRUCT S_RadialProfilerParams srp
	variable rmax
	
	switch( cba.eventCode )
		case 2: // mouse up
			strswitch(cba.ctrlName)
				case "checkCircular":
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy, srp.rinx,srp.aspyx,srp.circular)
					RP_SetOuter(srp.rdouter,srp.circular)
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					break
				case "checkShowBlob":
#ifndef DEMO	
					RP_ShowBlob(cba.checked)
#endif
					break
				endswitch
		case -1: // control being killed
			break
	endswitch

	return 0
end

Function RP_MoveInner(cx, cy, rinx, aspyx, circular)
	variable cx, cy, rinx, aspyx, circular
	
	variable rx, ry
	variable drx, dry
	variable left, top, right, bottom
	
	rx = Sf_GetRings(1)[2]
	// no rings set yet
	if (numtype(rx) == 2)
		rx = rinx
		ry = rinx
	else
		ry = Sf_GetRings(1)[3]
	endif
	
	if (circular)
		aspyx = 1
		ry = rx
	else
		ry = rx*aspyx
	endif
	
	drx = rinx - rx
	dry = drx*aspyx
	
	rx += drx
	ry += dry
	
	left = cx - rx
	right = cx + rx
	top = cy - ry
	bottom = cy + ry

	FPutCenter(cx,cy,1)
	FDrawOval(left, top, right, bottom, "innerring")
	Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,1)
	
	return 0
end

// set the outer ring
// mpcf - max expansion factor
// circular - circular or not
Function RP_SetOuter(mpcf, circular)
	variable mpcf, circular
	
	variable cx, cy, rx, ry
	variable left, top, right, bottom
	
	mpcf += 1
	
	cx = Sf_GetRings(1)[0]
	cy = Sf_GetRings(1)[1]
	rx = floor(mpcf*Sf_GetRings(1)[2])
	ry = floor(mpcf*Sf_GetRings(1)[3])
	
	left = cx - rx
	right = cx + rx
	top = cy - ry
	bottom = cy + ry

	FDrawOval(left, top, right, bottom, "outerring")
	Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,2)
	
	return 0
end

Function RP_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	variable cx, cy, rinx, aspyx
	string CList
	STRUCT S_RadialProfilerParams srp
	
	switch( ba.eventCode )
		case 2: // mouse up
			// click code here
			
			strswitch(ba.ctrlName)
				case "buttonautosetrings":
					// no calculations without threshold
					if (!f_ThresholdExists())
						DoAlert/T="Radial Profiler" 0, "Set a threshold first to auto-find the object for the rings."
						break
					endif
					// check that marquee has not disappeared in the meantime
					if (!f_hasMQ())
						PopupMenu popupRPMask, win=RP_Inputs, mode=1
					endif
					// put inner and outer rings
					Sf_ReadRadialProfilerParams(srp)
					aspyx = RP_SetTarget(srp.fedge,srp.circular,srp.mask)
					RP_SetOuter(srp.rdouter,srp.circular)
					// make the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// update the control panel
					CList = ControlNameList("RP_Inputs",";","setvar*")
					ModifyControlList CList, win=RP_Inputs, disable=0
					ValDisplay valdispAspYX, win=RP_Inputs, value=_NUM:aspyx, disable=0
					CheckBox checkcircular, win=RP_Inputs, disable=0
					PopupMenu popupRPMask,win=RP_Inputs, disable=0
					cx = Sf_GetRings(1)[0]
					SetVariable setvarcx, win=RP_Inputs, value=_NUM:cx
					cy = Sf_GetRings(1)[1]
					SetVariable setvarcy, win=RP_Inputs, value=_NUM:cy
					rinx = Sf_GetRings(1)[2]
					SetVariable setvarmir, win=RP_Inputs, value=_NUM:rinx
#ifndef DEMO
					PopupMenu popupmethod, win=RP_Inputs, disable=0
					Button buttonanalyze,win=RP_Inputs, disable=0
#endif					
					break
				case "buttonclearrings":
					// clear the control panel
					CList = ControlNameList("RP_Inputs",";","setvar*")
					ModifyControlList CList, win=RP_Inputs, disable=2
					CList = ControlNameList("RP_Inputs",";","valdisp*")
					ModifyControlList CList, win=RP_Inputs, disable=2
					CheckBox checkcircular, win=RP_Inputs, disable=2
					Button buttonanalyze,win=RP_Inputs, disable=2
					Button buttonpublish,win=RP_Inputs, disable=2
					Button buttonSave,win=RP_Inputs, disable=2
					Button buttonAutoSetRings,win=RP_Inputs, disable=0
					PopupMenu popupRPMask,win=RP_Inputs, disable=2, mode=1
					PopupMenu popupquadrant,win=RP_Inputs, disable=2, mode=1
					PopupMenu popupmethod, win=RP_Inputs, disable=2
					PopupMenu popupoptimize,win=RP_Inputs, mode=1, disable=2
					SetVariable setvarmir, win=RP_Inputs, value=_NUM:NaN, disable=2
					RP_ClearValDisplays()
					// clear the graphs
					FDrawOval(0,0,0,0,"innerring",kill=1)
					FDrawOval(0,0,0,0,"outerring",kill=1)
					FPutCenter(0,0,0)
					RP_UpdateRowColPlot(clear=1)
#ifndef DEMO
					RP_PlotNormalizedProfile(clear=1)
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "buttonanalyze":
#ifndef DEMO
					RP_AnalyzeProfile()
#endif
					break
				case "buttonsave":
#ifndef DEMO
					RP_SaveResults()
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
				case "buttonpublish":
#ifndef DEMO
					RP_PublishRadialProfile()
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
			endswitch
				break
		case -1: // control being killed
			break
	endswitch

	return 0
end

Function RP_PopMenuProc(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	STRUCT S_RadialProfilerParams srp

	switch( pa.eventCode )
		case 2: // mouse up
			strswitch(pa.ctrlname)
				case "popupQuadrant":
#ifndef DEMO
					RP_MakeRadialQuadrant(pa.popNum)
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
				case "popupMethod":
#ifndef DEMO
					RP_AnalyzeProfile()
#endif
					break
				case "popupoptimize":
#ifndef DEMO
					if (pa.popNum == 1)
						SetVariable setvarcx, win=RP_Inputs, disable=0
						SetVariable setvarcy, win=RP_Inputs, disable=0
						SetVariable setvarmir, win=RP_Inputs, disable=0
						Button buttonAutosetrings, win=RP_Inputs, disable=0
					else
						RP_OptimizeTargetCircle(pa.popNum - 1)
						SetVariable setvarcx, win=RP_Inputs, value=_NUM:Sf_GetRings(1)[0], disable=2
						SetVariable setvarcy, win=RP_Inputs, value=_NUM:Sf_GetRings(1)[1], disable=2
						SetVariable setvarmir, win=RP_Inputs, value=_NUM:Sf_GetRings(1)[2], disable=2
						Button buttonAutosetrings, win=RP_Inputs, disable=2
						Sf_ReadRadialProfilerParams(srp)
						RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)		
						RP_SetOuter(srp.rdouter,srp.circular)				
						RP_MakeRowColProfiles(srp.faveover)
						RP_UpdateRowColPlot()					
					endif
					PopupMenu popupquadrant,win=RP_Inputs, disable=2, mode=1
					Button buttonpublish,win=RP_Inputs, disable=2
					Button buttonSave,win=RP_Inputs, disable=2
					CheckBox checkShowBlob, win=RP_Inputs, value=0, disable=2
					RP_ClearValDisplays()
					// clear the graphs
					RP_UpdateRowColPlot()
					RP_PlotNormalizedProfile(clear=1)
					RP_MakeRadialQuadrant(1)
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
end

#ifndef DEMO
Function RP_AnalyzeProfile()

	STRUCT S_RadialProfilerValues rpv
	STRUCT S_RadialProfilerParams srp
	string CList
	
	if (!f_ThresholdExists())
		return 0
	endif
	// check that marquee has not disappeared in the meantime
	if (!f_hasMQ())
		PopupMenu popupRPMask, win=RP_Inputs, mode=1
	endif
	ControlInfo/W=RP_Inputs popupmethod
	RP_CalcRadialProfileResults(v_value)
	RP_PlotNormalizedProfile()
	Sf_GetRadialProfilerValues(rpv)
	Sf_DisplayRadialProfilerValues(rpv)
	RP_MakeRadialQuadrant(srp.overlay)
	CList = ControlNameList("RP_Inputs",";","valdisp*")
	ModifyControlList CList, win=RP_Inputs, disable=0
	Button buttonpublish,win=RP_Inputs, disable=0
	Button buttonSave,win=RP_Inputs, disable=0
	PopupMenu popupquadrant, win=RP_Inputs, disable=0			
	PopupMenu popupoptimize,win=RP_Inputs, disable=0
	CheckBox checkShowBlob, win=RP_Inputs, disable=0

	return 0
end
#endif